from django.db import models
from django.db.models.options import override

#from .views import SearchableView
# ^^^ raises Circular Import

class Keyword(models.Model):
    """
    this is the model that should contain searchable keywords
    """
    text = models.CharField(max_length=40)

class AbstractSearchable(models.Model):
    """
    Abstract class for any model that should be searchable.
    This model will be searched for by the searchbox that is in every upper part of the website.
    """
    title = models.CharField(max_length=50)
    subtitle = models.CharField(max_length=50)
    desc = models.CharField(max_length=250, unique=True)
    # may be empty/blank for some entries
    date = models.DateField(blank=True)
    keywords = models.ManyToManyField(Keyword)

    class Meta:
        """
        AbstractSearchable is an abstract model
        """
        abstract = True

    @classmethod
    def regenerate_all_entries(cls):
        """
        regenerate all searchable items
        """
        raise NotImplementedError

class StaticSite(AbstractSearchable):
    """
    This model represents any static site, such as start:index, 
    that should show up in search.

    Every searchable view should inherit from start.views.SearchableView.
    # TODO automate scanning for SearchableView classes
    """
    

    @override
    def regenerate_all_entries(cls):

        # this does not actually contain all views, only those that should show
        # up in the search.
        # TODO automate searching for these
        #all_views: list[SearchableView] = []

        raise NotImplementedError
